@@ -39,7 +39,8 @@
 #' @importFrom tibble tibble
 #' @importFrom survival coxph
 #' @importFrom survey svycoxph
-#' @importFrom stats confint coefficients anova
+#' @importFrom stats confint coefficients
+#' @importFrom car Anova
 #' @importFrom utils tail
 
 TableSubgroupCox <- function(formula, var_subgroup = NULL, var_cov = NULL, data, time_eventrate = 3 * 365, decimal.hr = 2, decimal.percent = 1, decimal.pvalue = 3){
@@ -47,9 +48,11 @@ TableSubgroupCox <- function(formula, var_subgroup = NULL, var_cov = NULL, data,
   . <- NULL
   
   if (any(class(data) == "survey.design" & !is.null(var_subgroup))){
-    if (is.numeric(data$variable[[var_subgroup]])) stop("var_subgroup must categorical.")
+    if (is.numeric(data$variables[[var_subgroup]])) stop("var_subgroup must categorical.")
+    if (length(unique(data$variables[[as.character(formula[[3]])]])) != 2) stop("Independent variable must have 2 levels.")
   } else if(any(class(data) == "data.frame" & !is.null(var_subgroup))){
     if (is.numeric(data[[var_subgroup]])) stop("var_subgroup must categorical.")
+    if (length(unique(data[[as.character(formula[[3]])]])) != 2) stop("Independent variable must have 2 levels.")
   }
   
   
@@ -64,18 +67,20 @@ TableSubgroupCox <- function(formula, var_subgroup = NULL, var_cov = NULL, data,
   possible_rowone <- purrr::possibly(function(x){x[1, ]}, NA)
   
   formula.km <- formula
-  var_cov <- setdiff(var_cov, var_subgroup)
+  var_cov <- setdiff(var_cov, c(as.character(formula[[3]]), var_subgroup))
   if (is.null(var_subgroup)){
     if (!is.null(var_cov)){
       formula <- as.formula(paste0(deparse(formula), " + ", paste(var_cov, collapse = "+")))
     }
     if (any(class(data) == "survey.design")){
       model <- survey::svycoxph(formula, design = data, x= T)
+      if (!is.null(model$xlevels) & length(model$xlevels[[1]]) != 2) stop("Categorical independent variable must have 2 levels.")
       res.kap <- survey::svykm(formula.km, design = data)
       prop <- round(100 * sapply(res.kap, function(x){1 - x[["surv"]][which.min(abs(x[["time"]] - time_eventrate))]}), decimal.percent)
       names(prop) <- model$xlevels[[1]]
     } else{
-      model <- survival::coxph(formula, data = data, x= T)
+      model <- survival::coxph(formula, data = data, x= TRUE)
+      if (!is.null(model$xlevels) & length(model$xlevels[[1]]) != 2) stop("Categorical independent variable must have 2 levels.")
       res.kap <- survival::survfit(formula.km, data = data)
       res.kap.times <- summary(res.kap, times = time_eventrate)
       prop <- round(100 * (1 - res.kap.times[["surv"]]), decimal.percent)
@@ -111,16 +116,19 @@ TableSubgroupCox <- function(formula, var_subgroup = NULL, var_cov = NULL, data,
       data$variables[[var_subgroup]] %>% table %>% names -> label_val
       label_val %>% purrr::map(~possible_svycoxph(formula, design = subset(data, get(var_subgroup) == .), x = TRUE)) -> model
       xlev <- survey::svycoxph(formula, design = data)$xlevels
-      xlabel <- names(xlev)[1]
+      xlabel <- names(attr(model[[which(!is.na(model))[1]]]$x, "contrast"))[1]
       pvs_int <- possible_svycoxph(as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula))), design = data) %>% summary %>% coefficients
       pv_int <- round(pvs_int[nrow(pvs_int), ncol(pvs_int)], decimal.pvalue)
-      if (length(xlev[[xlabel]]) > 2){
+      if (!is.null(xlev) & length(xlev[[1]]) != 2) stop("Categorical independent variable must have 2 levels.")
+      
+      
+      if (length(label_val) > 2){
         data.int <- data$variables
-        model.int <- survival::coxph(as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula))), data = data.int, weights = get(names(data$allprob)))
+        model.int <- survival::coxph(as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula))), data = data.int, weights = get(names(data$allprob)), robust =T)
         model.int$call$formula <- as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula)))
         model.int$call$data <- as.name("data.int")
-        pv_anova <- stats::anova(model.int)
-        pv_int <- pv_anova[nrow(pv_anova), 4]
+        pv_anova <- car::Anova(model.int, test.statistics = "Wald")
+        pv_int <- pv_anova[nrow(pv_anova), 3]
       }
       res.kap <- purrr::map(label_val, ~survey::svykm(formula.km, design = subset(data, get(var_subgroup) == . )))
       mkz <- function(reskap){
@@ -134,15 +142,17 @@ TableSubgroupCox <- function(formula, var_subgroup = NULL, var_cov = NULL, data,
       data %>% filter(!is.na(get(var_subgroup))) %>% group_split(get(var_subgroup)) %>% purrr::map(~possible_coxph(formula, data = ., x= T)) -> model
       data %>% filter(!is.na(get(var_subgroup))) %>% select(var_subgroup) %>% table %>% names -> label_val
       xlev <- survival::coxph(formula, data = data)$xlevels
-      xlabel <- names(xlev)[1]
+      xlabel <- names(attr(model[[which(!is.na(model))[1]]]$x, "contrast"))[1]
       model.int <- possible_coxph(as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula))), data = data)  
       pvs_int <- model.int %>% summary %>% coefficients
       pv_int <- round(pvs_int[nrow(pvs_int), ncol(pvs_int)], decimal.pvalue)
-      if (length(xlev[[xlabel]]) > 2){
+      if (!is.null(xlev) & length(xlev[[1]]) != 2) stop("Categorical independent variable must have 2 levels.")
+      
+      if (length(label_val) > 2){
         model.int$call$formula <- as.formula(gsub(xlabel, paste(xlabel, "*", var_subgroup, sep=""), deparse(formula)))
         model.int$call$data <- as.name("data")
-        pv_anova <- stats::anova(model.int)
-        pv_int <- round(pv_anova[nrow(pv_anova), 4], decimal.pvalue)
+        pv_anova <- car::Anova(model.int, test.statistics = "Wald")
+        pv_int <- round(pv_anova[nrow(pv_anova), 3], decimal.pvalue)
       }
       res.kap.times <- data %>% filter(!is.na(get(var_subgroup))) %>% group_split(get(var_subgroup)) %>% purrr::map(~survival::survfit(formula.km, data = .)) %>% purrr::map(~summary(., times = time_eventrate))
       prop <- res.kap.times %>% purrr::map(~round(100 * (1 - .[["surv"]]), decimal.percent)) %>% dplyr::bind_cols() %>% t
